﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data;
using Produmex.Sbo.Logex.HarvardEngineering.BusinessObjects;
using Produmex.Sbo.Logex.HarvardEngineering.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.HarvardEngineering.Providers;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    public class WorkflowScript_AfterLogisticUnitFullHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<int> LUID;
        
        // Sub-flows
        // <none>


        public WorkflowScript_AfterLogisticUnitFullHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            Message msg = null;

            string query = null;
            string boxType = null;
            string initialErrorKey = null;
            string boxCode = null;

            int qtyBox = 0;

            bool goBack = false;

            Dictionary<string, int> boxCodes = new Dictionary<string, int>();

            DataSet dsBoxTypes = null;

            Step_SelectBoxType:

            #region SELECT BOX TYPE

           

            query = BuildQuery.GetAllBoxTypes();
            dsBoxTypes = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_HRV_SELECT_BOX_TYPES",
                "AllowToGoBack", false,
                "StringValueDS", dsBoxTypes));
            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;


            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                boxType = ExtractParameter<string>(msg.Parameters, "stringValue");
                boxCode = (dsBoxTypes.Tables[0].Select("stringValue = '" + boxType + "'")[0][PmxHarvardBoxTypeTable.Columns.Code.NAME]).ToString();

                goto Step_EnterNumberOfBox;
            }
            else
            {
                //goBack = true;
                return;
            }

            #endregion

            Step_EnterNumberOfBox:

            #region ENTER NUMBER OF BOX

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                 DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_HRV_ENTER_BOX",
                "AllowToGoBack", true,
                "InitialQuantity", 1,
                "NumberOfDigitsAfterDecimal", 0,
                "MaximumQuantity", -1,
                "UOM", ""     
                ));

            //clear data
            goBack = false;
            initialErrorKey = null;
            msg = WaitForMessage();
            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                qtyBox= Convert.ToInt32(ExtractParameter<double>(msg.Parameters, "quantity"));
                goto Step_EnterNextBoxType;
            }
            else
            {
                goBack = true;
                goto Step_SelectBoxType;
            }

            #endregion

            Step_EnterNextBoxType:

            #region ENTER NEXT BOX TYPE

            if (boxCodes.ContainsKey(boxCode))
            {
                boxCodes[boxCode] = qtyBox;
            }
            else
            {
                boxCodes.Add(boxCode, qtyBox);
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                    "TitleKey", "MSG_TITLE_HVR_NEW_BOX_TYPE",
                    "MessageKey", "MSG_QUESTION_HVR_NEW_BOX_TYPE"));
            msg = WaitForMessage();
            goBack = false;
            if (msg.Name.ToLower().EndsWith(".yes"))
            {
                goto Step_SelectBoxType;
            }

            goto Step_FillInData;
          
            #endregion
            Step_FillInData:

            #region FILL IN DATA

            foreach (KeyValuePair<string, int> pair in boxCodes)
            {

                try
                {

                    sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate (PmxDbConnection conn, object[] parameters)
                    {
                        PmxLinkLuidBoxTypeProvider prov = new PmxLinkLuidBoxTypeProvider(conn);
                        PmxLinkLuidBoxType luidBoxType = prov.GetNewBO();

                        //int? Luid = Convertor.ConvertToInt32(parameters[0]);
                        //string boxCode = parameters[1].ToString();
                        //int quantity = Convertor.ConvertToInt32(parameters[2]);

                        int? Luid = LUID;
                        string tempBoxCode = pair.Key;
                        int quantity = pair.Value;


                        luidBoxType.BoxCode = tempBoxCode;
                        luidBoxType.Quantity = quantity;
                        luidBoxType.LogisticUnitIdentKey = Luid;

                        prov.AddBO(luidBoxType);

                        return null;
                    });
                   
                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (ex is ProdumexException)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();
                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                                "NoTranslationOfMessageKey", true,
                                "IsError", true,
                                "ShowButton", true));
                        WaitForMessage();
                    }

                    //ask user if he wants to try again after error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                            "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_FillInData;
                    }
                }
            }

            return;
            #endregion
        }

        #endregion

        #region BUILD QUERY

       
            /// <summary>
            /// Static class grouping our native queries
            /// </summary>
        internal static class BuildQuery
        {

            /// <summary>
            /// Gets all box types.
            /// </summary>
            /// <returns></returns>
            public static string GetAllBoxTypes()
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxHarvardBoxTypeTable.NAME, PmxHarvardBoxTypeTable.Columns.Name.NAME, StringValueDataSet.Columns.STRING_VALUE.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxHarvardBoxTypeTable.NAME, PmxHarvardBoxTypeTable.Columns.Code.NAME);

            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxHarvardBoxTypeTable.NAME));

            return strBuilder.ToString();
        }

        }

        #endregion


    }


}
