﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;


// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// Hook flow for after a Logistic unit is received.
    /// This flow is called after the print, but BEFORE it is actually added to the database!
    /// </summary>
    /// <seealso cref="Produmex.Foundation.Workflows.WorkflowInstanceScriptBase" />
    public class WorkflowScript_AfterLogisticUnitReceivedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadWriteBinder<List<LogisticUnitGoodsReceipt>> LogisticUnits;

        // Sub-flows
        // <none>



        /// <summary>
        /// Gets or sets the allowed locations.
        /// </summary>
        /// <value>
        /// The allowed locations.
        /// </value>
        private List<LogisticUnitGoodsReceipt> logisticUnits
        {
            get { return LogisticUnits.Get(); }
            set { LogisticUnits.Set(value); }
        }


        ISboProviderService m_sboProviderService;

        public WorkflowScript_AfterLogisticUnitReceivedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
          

            string qualityStatusCode = null;
            bool isValid = false;
            PmxItemInfo itemInfo = null;

           m_sboProviderService.InvokeMethodWithDbConnection<bool>(false, true, null, null, delegate (PmxDbConnection conn, object[] parameters)
            {
                foreach (LogisticUnitGoodsReceipt item in logisticUnits)
                {

                    PmxItemAllConnectionsProvider itemProv = new PmxItemAllConnectionsProvider(conn);
                    qualityStatusCode = itemProv.GetQualityStatusCodeReception(item.ItemsOnLogisticUnit[0].ItemCode, item.ItemsOnLogisticUnit[0].BatchNumber1, ref itemInfo);
                    if (qualityStatusCode != PmxQualityStatusCodes.RELEASED_CODE)
                    {
                        isValid = true;
                        break;

                    }
                }

                return isValid;
            });
            
        
            if (isValid)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                   "MessageKey", "MSG_PUT_INSPECTION_AREA",
                       "NoTranslationOfMessageKey", false,
                       "ShowButton", true));
                WaitForMessage();

            }



            }

        #endregion

        #region HELPERS
        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }
        #endregion

        #region BUILD QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {

            /// <summary>
            /// Gets the stock of luids.
            /// </summary>
            /// <param name="listOfLUID">The list of LUIDs.</param>
            /// <returns></returns>
            internal static string GetStockLogisticUnits(IList<int> listOfLUID, string status )
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT * ");
             
                strBuilder.AppendFormat("\r\n FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));

                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

                string operand = "\r\n WHERE ";

                if (!string.IsNullOrEmpty(status))
                {
                    strBuilder.Append(operand);
                    strBuilder.AppendFormat(" {0} <> {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(status, CultureInfo.InvariantCulture));
                    operand = "\r\n AND ";
                }


                if (listOfLUID != null && listOfLUID.Count > 0)
                {
                    strBuilder.Append(operand);
                    strBuilder.AppendFormat(" {0} IN ({1})",
                        SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(listOfLUID, CultureInfo.InvariantCulture));
                    operand = "\r\n AND ";
                }

                return strBuilder.ToString();
            }
        }

        #endregion
    }


}
